' esp8266 Library version 1.0.00
' Written by Rich Martin (datawiz) August 2020

' **************************************************
' * WIFI LIBRARY 
' **************************************************

' connect to an AP using SSID$ and PASS$
sub esp.wifi.begin(SSID$,PASS$)
  esp.var.return$ = ""
  esp.var.timeout% = 2000
  esp.waitfor("AT+CWJAP_CUR="+chr$(34)+SSID$+chr$(34)+","+chr$(34)+PASS$+chr$(34),"OK",esp.waitfor.mode.match,1)
  if instr(1,esp.var.waitfor.buffer$, "CONNECTED") <> 0 then esp.var.return$ = "CONNECT AP"
  if instr(1,esp.var.waitfor.buffer$, "GOT IP") <> 0 then esp.var.return$ = "CONNECT IP"
  if esp.var.return$ = "" then esp.var.return$ = "ERROR"
end sub

' get current connected ssid
sub esp.wifi.ssid()
  esp.var.return$ = ""
  esp.waitfor("AT+CWJAP_CUR?","OK",esp.waitfor.mode.match,1)
  if esp.var.debug% = 1 then print esp.var.waitfor.buffer$
  ' parse this
  ' +CWJAP_CUR:<ssid>,<bssid>,<channel>,<rssi>
  ' OK
end sub

' get current connected bssid
sub esp.wifi.bssid()
  esp.var.return$ = ""
  esp.waitfor("AT+CWJAP_CUR?","OK",esp.waitfor.mode.match,1)
  if esp.var.debug% = 1 then print esp.var.waitfor.buffer$
  ' parse this
  ' +CWJAP_CUR:<ssid>,<bssid>,<channel>,<rssi>
  ' OK
end sub

' get current connection encryption information
sub esp.wifi.encryptiontype(SSID$)
  esp.var.return$ = ""
  esp.waitfor("AT+CWLAP="+chr$(34)+SSID$+chr$(34),"OK",esp.waitfor.mode.match)
  if esp.var.debug% = 1 then print esp.var.waitfor.buffer$
  ' parse this
  ' +CWLAP:<ecn>,<ssid>,<rssi>,<mac>,<channel>,<freq offset>,<freqcali>,<pairwise_cipher>,<group_cipher>,<bgn>,<wps>
  ' OK
end sub

' disconnect from AP
sub esp.wifi.disconnect()
  esp.var.return$ = ""
  esp.waitfor("AT+CWQAP","OK",esp.waitfor.mode.match,1)
end sub

' configure IP, Gateway, and Netmask
sub esp.wifi.config(IP$, GATEWAY$, NETMASK$)
  esp.var.return$ = ""
  esp.waitfor("AT+CIPSTA_DEF="+chr$(34)+IP$+chr$(34)+","+chr$(34)+GATEWAY$+chr$(34)+","+chr$(34)+NETMASK$+chr$(34),"OK",esp.waitfor.mode.match,1)
end sub

' configure dns name server
sub esp.wifi.dns(MODE$,DNS$)
  esp.var.return$ = ""
  if MODE$ = "OFF" then esp.waitfor("AT+CIPDNS_DEF=0","OK",esp.waitfor.mode.match,1)
  if MODE$ = "ON"  then esp.waitfor("AT+CIPDNS_DEF=1,"+chr$(34)+DNS$+chr$(34),"OK",esp.waitfor.mode.match,1)
end sub

' configure dhcp
sub esp.wifi.dhcp(MODE$)
  esp.var.return = ""
  if MODE$ = "OFF" then esp.waitfor("AT+CWDHCP_DEF=1,0","OK",esp.waitfor.mode.match,1)
  if MODE$ = "ON"  then esp.waitfor("AT+CWDHCP_DEF=1,1","OK",esp.waitfor.mode.match,1)
end sub

' scan available wifi networks
sub esp.wifi.scannetworks()
  esp.var.return$ = ""
  esp.waitfor("AT+CWLAP","OK",esp.waitfor.mode.match,1)
  if esp.var.debug% = 1 then print esp.var.waitfor.buffer$
end sub

' get esp mac address
sub esp.wifi.macaddress()
  local x%, y%
  local mac$
  esp.var.return$ = ""
  esp.waitfor("AT+CIFSR","OK",esp.waitfor.mode.match,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.wifi.macaddress:"+esp.var.return$ : end sub
  x% = instr(1,esp.var.waitfor.buffer$,"STAMAC,")
  x% = x% + 8
  y% = instr(x%, esp.var.waitfor.buffer$,chr$(34))
  mac$ = mid$(esp.var.waitfor.buffer$,x%,y%-x%)
  esp.var.waitfor.buffer$ = mac$
end sub

' restart the esp8266
sub esp.wifi.restart
  print #esp.var.com.filenum%,"AT+RST"
  pause 2000
end sub

' send the escape sequence and an AT command immediately afterwards
sub esp.wifi.escape(ATCMD$)
  pause 1000
  print #esp.var.com.filenum%,"+++";
  pause 200
  if ATCMD$ <> "" then print #esp.var.com.filenum%,ATCMD$
end sub


' **************************************************
' * TCP CLIENT LIBRARY
' **************************************************

' connect to a server, null on success
sub esp.tcp.client.connect(IP$, PORT$)
  esp.var.return$ = ""
  ' Wifi station mode
  esp.waitfor("AT+CWMODE_CUR=1","OK",esp.waitfor.mode.match,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.client.connect 1:"+esp.var.return$ : end sub
  pause 200
  ' single connection mode
  esp.waitfor("AT+CIPMUX=0","OK",esp.waitfor.mode.match,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.client.connect 2:"+esp.var.return$ : end sub
  pause 200
  ' start a connection
  esp.waitfor("AT+CIPSTART="+chr$(34)+"TCP"+chr$(34)+","+chr$(34)+IP$+CHR$(34)+","+PORT$,"CONNECT",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.client.connect 3:"+esp.var.return$ : end sub
  pause 200
  ' set transparent mode
  esp.waitfor("AT+CIPMODE=1","OK",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.client.connect 4:"+esp.var.return$ : end sub
  pause 200
  ' start sending in transparent mode
  esp.waitfor("AT+CIPSEND","OK",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.client.connect 5:"+esp.var.return$ : end sub
  pause 200
end sub

' disconnect from a server, null on success
sub esp.tcp.client.stop()
  esp.var.return$ = ""
  esp.waitfor("AT+CIPCLOSE","CLOSED",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.client.stop:" + esp.var.return$
end sub

' ping a host, esp.var.return$ holds status
sub esp.tcp.client.ping(IP$)
  esp.var.return$ = ""
  esp.waitfor("AT+PING="+chr$(34)+IP$+chr$(34),"OK",esp.waitfor.mode.match,1)
  prints(esp.var.return$)
  '+TIME if successful
  '+timeout and ERROR if failed
end sub

' send a single byte
sub esp.tcp.client.write(BYTE$)
  esp.var.return$ = ""
  print #esp.var.com.filenum%, left$(BYTE$,1)
end sub

' is data available in the input buffer?, esp.tcp.client.available% returns number of bytes in queue
function esp.tcp.client.available() as integer
  esp.var.return$ = ""
  esp.tcp.client.available% = str$(loc(esp.var.filenum))
end function

' read a byte from the input buffer
function esp.tcp.client.read() as integer
  esp.var.return$ = ""
  esp.tcp.client.read% = val(input$(1,esp.var.filenum))
end function

' send data to the output buffer
sub esp.tcp.client.print(BYTES$)
  esp.var.return$ = ""
  print #esp.var.com.filenum%, BYTES$;
end sub

' send data to the output buffer, ending with CRLF
sub esp.tcp.client.println(BYTES$)
  esp.var.return$ = ""
  print #esp.var.com.filenum%, BYTES$
  if esp.var.crlf% = 1 then print #esp.var.com.filenum%, chr$(13)
end sub


' **************************************************
' * TCP SERVER LIBRARY
' **************************************************

' start a tcp server on a port
sub esp.tcp.server.begin(PORT$)
  esp.var.return$ = ""
  ' Wifi station mode
  esp.waitfor("AT+CWMODE_CUR=1","OK",esp.waitfor.mode.match,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.server.begin 1:"+esp.var.return$ : end sub
  pause 200
  ' multiple connection mode
  esp.waitfor("AT+CIPMUX=1","OK",esp.waitfor.mode.match,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.server.begin 2:"+esp.var.return$ : end sub
  pause 200
  ' start the server
  esp.waitfor("AT+CIPSERVER=1,"+PORT$,"OK",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.server.begin 5:"+esp.var.return$ : end sub
  pause 200
end sub

' stop the server
sub esp.tcp.server.stop
   esp.waitfor("AT+CIPSERVER=0","OK",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.tcp.server.stop:"+esp.var.return$ : end sub
  pause 200
end sub

' send a byte to the output buffer
sub esp.tcp.server.write(BYTE$)
  local OUT$
  esp.var.return$ = ""
  OUT$ = "AT+CIPSEND="+str$(esp.var.tcp.server.con%)+","+"1"
  esp.waitfor(OUT$,"SEND OK",esp.waitfor.mode.match, 1)
  if esp.var.return$ <> "" then 
    print "esp.tcp.server.write error: "+esp.var.return$
  else
    esp.waitfor(left$(BYTE$,1),"SEND OK",esp.waitfor.mode.match,0)    
  end if
end sub

' read a byte from a the input buffer
function esp.tcp.server.read()
  esp.var.return$ = ""
  esp.tcp.server.read% = val(input$(1,esp.var.filenum))
end function

' send bytes to the output buffer
sub esp.tcp.server.print(CONN%,BYTES$)
  local OUT$
  local ECHO%
  ECHO% = esp.var.echo%
  esp.var.echo% = 0
  esp.var.return$ = ""
  OUT$ = "AT+CIPSEND="+str$(CONN%)+","+str$(len(BYTES$))
  esp.waitfor(OUT$,"OK",esp.waitfor.mode.match, 1)
  if esp.var.return$ <> "" then 
    print "esp.tcp.server.print error: "+esp.var.return$
  else
    esp.waitfor(BYTES$,"SEND OK",esp.waitfor.mode.match,0)
  end if
  esp.var.echo% = ECHO%
end sub

' send bytes to the output buffer, ending with CRLF
sub esp.tcp.server.println(CONN%,BYTES$)
  local OUT$
  local ECHO%
  BYTES$ = BYTES$ + chr$(13) + chr$(10)
  ECHO% = esp.var.echo%
  esp.var.echo% = 0
  esp.var.return$ = ""
  OUT$ = "AT+CIPSEND="+str$(CONN%)+","+str$(len(BYTES$))
  esp.waitfor(OUT$,"OK",esp.waitfor.mode.match, 1)
  if esp.var.return$ <> "" then 
    print "esp.tcp.server.print error: "+esp.var.return$
  else
    esp.waitfor(BYTES$,"SEND OK",esp.waitfor.mode.match,0)
  end if
  esp.var.echo% = ECHO%
end sub


' **************************************************
' * IP LIBRARY
' **************************************************

' get the currently assigned IP address, returned in esp.var.waitfor.buffer$
sub esp.ip.localip
  local x%, y%
  local ip$
  esp.var.return$ = ""
  esp.waitfor("AT+CIPSTA_CUR?","OK",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.ip.localip:"+esp.var.return$ : end sub
  x% = instr(1,esp.var.waitfor.buffer$,chr$(34))
  y% = instr(x%+1, esp.var.waitfor.buffer$,chr$(34))
  ip$ = mid$(esp.var.waitfor.buffer$,x%+1,y%-x%-1)
  esp.var.waitfor.buffer$ = ip$
end sub

' get the currently assigned netmask, returned in esp.var.waitfor.buffer$
sub esp.ip.netmask
  local x%, y%
  local nm$
  esp.var.return$ = ""
  esp.waitfor("AT+CIPSTA_CUR?","OK",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.ip.localip:"+esp.var.return$ : end sub
  x% = instr(1,esp.var.waitfor.buffer$,"netmask")
  x% = x% + 7 + 2
  y% = instr(x%, esp.var.waitfor.buffer$,chr$(34))
  nm$ = mid$(esp.var.waitfor.buffer$,x%,y%-x%)
  esp.var.waitfor.buffer$ = nm$
end sub

' get the currently assigned gateway, returned in esp.var.waitfor.buffer$
sub esp.ip.gateway
  local x%, y%
  local gw$
  esp.var.return$ = ""
  esp.waitfor("AT+CIPSTA_CUR?","OK",esp.waitfor.mode.match%,1)
  if esp.var.return$ <> "" then esp.var.return$="esp.ip.localip:"+esp.var.return$ : end sub
  x% = instr(1,esp.var.waitfor.buffer$,"gateway")
  x% = x% + 7 + 2
  y% = instr(x%, esp.var.waitfor.buffer$,chr$(34))
  gw$ = mid$(esp.var.waitfor.buffer$,x%,y%-x%)
  esp.var.waitfor.buffer$ = gw$
end sub

' **************************************************
' * LOW LEVEL LIBRARY
' **************************************************

' esp.waitfor() send a string to the remote host, and wait for an input based on a MODE
sub esp.waitfor(OUT$,EXPECT$,MODE%,NL%)
  local TIMESTART%
  esp.var.return$ = ""
  if esp.var.debug% then prints(OUT$)
  esp.var.waitfor.match$ = EXPECT$
  esp.var.waitfor.mode% = MODE%
  if OUT$  <> "" then print #esp.var.com.filenum%, OUT$;
  if NL% = 1 then print #esp.var.com.filenum%, chr$(13)
  if MODE% = esp.waitfor.mode.normal% then end sub
  esp.var.waitfor.done% = 1
  TIMESTART% = epoch(NOW)
  do while esp.var.waitfor.done%  
    if epoch(NOW) > TIMESTART% + 10 then esp.var.return$ = "waitfor Timeout" : exit
  loop
end sub

' set the echo status on the esp8266
sub esp.echo.set(MODE$)
  esp.var.return$ = ""
  select case lcase$(MODE$)
    case "off"
      esp.waitfor("ATE0","OK",esp.waitfor.mode.match,1)
    case "on"
      esp.waitfor("ATE1","OK",esp.waitfor.mode.match,1)
    case else
      esp.var.return$ = "esp.wifi.echo: Bad Option ON|OFF"
  end select
end sub

' clear any data in the input buffer. probably a better way to do this...
sub esp.buffer.clear
  local B$ = ""
  local X%
  for X% = 1 to 10  
    do while loc(#esp.var.com.filenum%) <> 0    
      B$=input$(255,#esp.var.com.filenum%)
    loop
  next X%
end sub

' main routine to handle data coming in from the input buffer, supports seveal modes (normal, match, # of bytes, mux)
sub esp.input.loop()
  local INCOUNT%
  local TOTALBYTES%
  local B$
  local BUFFCOUNT%
  
  select case esp.var.waitfor.mode%

    case esp.waitfor.mode.normal:
      blinkflag% = 1		
      B$ = input$(1,#esp.var.com.filenum%)
      if B$ <> "" then
        if soundflag% = 1 then 
          play tone RECVCLICK%, RECVCLICK%, RECVCLICKDELAY1%
          pause RECVCLICKDELAY2%
        end if
        if esp.var.echo% = 1 then prints(B$)
      end if
			blinkflag% = 0
			
    case esp.waitfor.mode.match:
      esp.var.return$ = ""
      esp.var.waitfor.buffer$ = ""
      B$ = ""
      INCOUNT% = 0
      TIMER = 0
      do
        B$ = input$(1,#esp.var.com.filenum%)
        if B$ <> "" then
          TIMER = 0
          esp.var.waitfor.buffer$ = esp.var.waitfor.buffer$ + B$
          if instr(1,esp.var.waitfor.buffer$,esp.var.waitfor.match$) <> 0 then exit
          if INCOUNT% >= 254 then esp.var.return$ = "match buffer overflow" : exit
          INCOUNT% = INCOUNT% + 1
        end if
        if TIMER >= esp.var.timeout% then esp.var.return$ = "match timeout" : exit
      loop
      esp.var.waitfor.match$ = ""
      esp.var.waitfor.done% = 0
      esp.var.waitfor.mode% = esp.waitfor.mode.normal

  case esp.waitfor.mode.mux
    esp.var.return$ = ""
    esp.var.waitfor.buffer$ = ""
      B$ = ""
      INCOUNT% = 0
      TIMER = 0
      do
        B$ = input$(1,#esp.var.com.filenum%)
        if B$ <> "" then
          TIMER = 0
          esp.var.waitfor.buffer$ = esp.var.waitfor.buffer$ + B$
          if instr(1,esp.var.waitfor.buffer$,"+IPD,") <> 0 then exit
          if INCOUNT% >= 254 then esp.var.return$ = "buffer overflow waiting mux" : exit
          INCOUNT% = INCOUNT% + 1
        end if
        if TIMER >= esp.var.timeout% then esp.var.return$ = "match timeout mux" : exit
      loop

      ' continue if we got the match
      if esp.var.return$ = "" then
        ' next in the queue is the connection number, a single character
        B$ = input$(1,#esp.var.com.filenum%)
        esp.var.tcp.server.con% = val(B$)
        ' skip the next character, which is a comma
        B$ = input$(1,#esp.var.com.filenum%)
       
        ' get the length of the payload
        esp.var.waitfor.buffer$ = ""
        INCOUNT% = 0
        TIMER = 0
        do
          B$ = input$(1,#esp.var.com.filenum%)
          if B$ <> "" then
            TIMER = 0
            if B$ = ":" then exit
            esp.var.waitfor.buffer$ = esp.var.waitfor.buffer$ + B$
            if INCOUNT% >= 254 then esp.var.return$ = "buffer overflow waiting mux 2" : exit
            INCOUNT% = INCOUNT% + 1
            if TIMER >= esp.var.timeout% then esp.var.return$ = "match timeout mux 2" : exit
          end if
        loop
        esp.var.tcp.server.len% = val(esp.var.waitfor.buffer$)
      end if
      esp.var.waitfor.match$ = ""

      if esp.var.return$ = "" then
        TOTALBYTES% = 0
        TIMER = 0
        longstring clear esp.var.waitfor.buffer.big%()
        do
          BUFFCOUNT% = loc(#esp.var.com.filenum%)
          if BUFFCOUNT% > 0 then
            if BUFFCOUNT% >= 200 then BUFFCOUNT% = 200      ' dont overflow our string
            B$ = input$(BUFFCOUNT%,#esp.var.com.filenum%)
            longstring append esp.var.waitfor.buffer.big%(), B$
            TOTALBYTES% = TOTALBYTES% + LEN(B$)
            esp.var.bytecounter% = TOTALBYTES%
            if TOTALBYTES% >= esp.var.tcp.server.len% then exit
            TIMER = 0
          end if
          if TIMER >= esp.var.timeout% then esp.var.return$ = "get num bytes timeout" : exit
        loop      
      end if
      esp.var.waitfor.done% = 0
      esp.var.waitfor.mode% = esp.waitfor.mode.normal

    case else:
      esp.var.return$ = ""
      TOTALBYTES% = 0
      TIMER = 0
      longstring clear esp.var.waitfor.buffer.big%()
      do
        BUFFCOUNT% = loc(#esp.var.com.filenum%)
        if BUFFCOUNT% > 0 then
          if BUFFCOUNT% >= 200 then BUFFCOUNT% = 200      ' dont overflow our string
          B$ = input$(BUFFCOUNT%,#esp.var.com.filenum%)
          longstring append esp.var.waitfor.buffer.big%(), B$
          TOTALBYTES% = TOTALBYTES% + LEN(B$)
          esp.var.bytecounter% = TOTALBYTES%
          if TOTALBYTES% >= esp.var.waitfor.mode% then exit
          TIMER = 0
        end if
        if TIMER >= esp.var.timeout% then esp.var.return$ = "get num bytes timeout" : exit
      loop      
      esp.var.waitfor.done% = 0
      esp.var.waitfor.mode% = esp.waitfor.mode.normal

  end select  
end sub
