' getfile.bas version 1.0.00 - written by Rich Martin (datawiz) August 2020
' use with getfile.py on a computer
'
option explicit
option default none

#include "esp8266var.inc"

const CHUNKSIZE = 1400              ' number of bytes the server sends to client. Make sure it matches in the server

dim SERVER_IP$  = "192.168.1.100"   ' server ip, hostnames also work
dim SERVER_PORT$= "6000"            ' server port

dim COM_PORT$ = "COM2"              ' CMM2 COM port the esp is wired up to
dim COM_SPEED$ = "115200"           ' COM port speed setting, needs to match the esp (115200 seems to be the default
dim COM_BUFFSIZE$ = "256"           ' COM port buffer size (256 is the max)

dim TOTALFS% = 0                    ' total file size returned from the server
dim FILENAME$ = ""                  ' static file name, will also be replaced by command line arg
dim TIMESTART%, TIMEEND%            ' variables for determining the elapsed time for the xfer
dim CHUNK%, SIZECOUNT%, RETRY%, BPS%, ESTIMATE%     ' variables for main xfer loop

print MM.CMDLINE$
if MM.CMDLINE$ = "" then
  print "Error, missing filename. Try:"
  print "  run ";chr$(34);"getfile.bas";chr$(34);", <filename to get from server>"
  print "    -or-"
  print "  *getfile <filename to get from server>
  end
end if

if SERVER_IP$ = "" or SERVER_PORT$ = "" then
  print "Server and/or Port was not found."
  print "Error, please edit program and define these variables"
  end
end if

FILENAME$ = "bars.bmp"

cls
esp.var.com.filenum% = 5      ' set the com file number 
open COM_PORT$+":"+COM_SPEED$+","+COM_BUFFSIZE$+",esp.input.loop" as esp.var.com.filenum%   ' set up COM port, speed, and set the interrupt for recv data to the esp8266 library routine
esp.var.echo% = 0                                   ' Dont print anything we recv in esp.input.loop
esp.waitfor("AT","OK",esp.waitfor.mode.match,1)                 ' Test to see what mode we are in
if esp.var.return$ <> "" then esp.wifi.escape("AT+CIPCLOSE") ' We are in transparent mode, need to escape out and disconnect from any connection
esp.echo.set("OFF")                                      ' esp will not echo back what we send to it
esp.buffer.clear                                         ' flush out the esp recv buffer in case anything was in there
pause 200
esp.tcp.client.connect(SERVER_IP$, SERVER_PORT$)                ' open TCP connection to the server
if esp.var.return$ = "" then print "Connected OK"               ' connection process is a success
if esp.var.return$ <> "" then print esp.var.return$ : goto TheEnd  ' connection process failed, exit
pause 200

esp.waitfor("SEND "+FILENAME$,"|",esp.waitfor.mode.match,0)         ' xmit the SEND command to server, and include filename. We expect a return from server that has the file size
if esp.var.return$ <> "" then print esp.var.return$ : goto TheEnd  ' if there was an error getting a response, then exit
TOTALFS% = VAL(left$(esp.var.waitfor.buffer$,LEN(esp.var.waitfor.buffer$)-1)) ' determine the returned file size
if TOTALFS% = 0 then print "Server: File Not Found" : goto TheEnd  ' if the file size was 0 that means file not found on the server

ESTIMATE% = ((TOTALFS%*8)/(VAL(COM_SPEED$)*.61))    ' 61% of max port speed is what I use for an estimate
print TOTALFS%;" bytes   estimated in";ESTIMATE%;" seconds"
open FILENAME$ for output as #6

esp.var.timeout% = 4000             ' increase timeout for bad wifi reception
if TOTALFS% >= CHUNKSIZE then   ' determine initial CHUNK size based on size of incoming file
  CHUNK% = CHUNKSIZE            ' use the normal CHUNKSIZE
else
  CHUNK% = TOTALFS%             ' the incoming file is smaller that CHUNKSIZE, so reduce it to match
end if

SIZECOUNT% = TOTALFS%           ' load SIZECOUNT for main loop, will count down to 0
TIMESTART% = epoch(NOW)         ' set up the start time of the transfer
RETRY% = 3                      ' retry attempts to server if there is a timeout
do while SIZECOUNT% > 0
  esp.var.waitfor.mode% = CHUNK%        ' set the amount of bytes we want to receive
  esp.var.waitfor.done% = 1             ' this flag signals when the bytes are read and we can continue in this te
    do while esp.var.waitfor.done%      ' loop waiting for esp.input.loop (mode # bytes) is complete
    loop
  if esp.var.return$ <> "" then                                   ' We had an error, tell server to resend this chunk
    if RETRY% = 0 then 
      print "Wifi problem! 3 retries attempted:";esp.var.return$
      esp.waitfor("ER","",esp.waitfor.mode.normal,0)                  ' Send ER to server to exit
      goto TheEnd                                            ' time to exit out due to errors
    end if
    RETRY% = RETRY% - 1
'    esp.waitfor("RS","",esp.waitfor.mode.normal,0)                    ' Send RS to server to resend 
  end if
  longstring print #6,esp.var.waitfor.buffer.big%();                    ' write the longstring to file
  longstring clear esp.var.waitfor.buffer.big%()                        ' clear it for next chunk
  SIZECOUNT% = SIZECOUNT% - CHUNK%                            ' subtract last chunk from SIZECOUNT
  if SIZECOUNT% < CHUNKSIZE then CHUNK% = SIZECOUNT%          ' the last chunk is whatever remains
'  esp.waitfor("OK","",esp.waitfor.mode.normal,0)                      ' Send OK to server to get next chunk
  esp.tcp.client.print("OK")                      ' Send OK to server to get next chunk
loop
TIMEEND% = epoch(NOW)                                         ' end of transfer
if TIMEEND% - TIMESTART% = 0 then TIMEEND% = TIMEEND% + 1     ' avoid divide by zero error on small xfer
print TOTALFS%;" bytes transferred in"TIMEEND%-TIMESTART%;" seconds ";  ' calculate stats from xfer
BPS% = ((TOTALFS%/(TIMEEND%-TIMESTART%))*8) : print BPS%" bps"
close #6                          ' close the file handle for downloaded file
TheEnd:
esp.wifi.escape("AT+CIPCLOSE")     ' escape transparent mode with +++ and then tell esp to close the socket
pause 2000
close esp.var.com.filenum%
end

#include "esp8266lib.inc"
