' setupwifi.bas - connect the esp8266 to an Access Point using SSID and password
' written by Rich Martin (datawiz) August 2020
'
option explicit
option default none

#include "esp8266var.inc"

dim COM_PORT$ = "COM2"              ' cmm2 COM port the esp is wired up to
dim COM_SPEED$ = "115200"           ' COM port speed setting, needs to match the esp (115200 seems to be the default
dim COM_BUFFSIZE$ = "256"           ' COM port buffer size (256 is the max)
dim SERVER_IP$  = ""  ' server ip, hostnames also work
dim SERVER_PORT$= ""            ' server port

dim SSID$ = ""
dim SSIDPW$ = ""
dim MYIP$ = ""
dim CONTINUE$ = ""
dim X%, Y%

esp.var.com.filenum% = 5
open COM_PORT$+":"+COM_SPEED$+","+COM_BUFFSIZE$+",esp.input.loop" as esp.var.com.filenum%

START:
  cls
  esp.var.echo% = 0
  esp.waitfor("AT+RST","",esp.waitfor.mode.normal,1)  ' reset esp
  pause 1000
  esp.waitfor("AT+CWQAP","OK",esp.waitfor.mode.match,1) ' disconnect from ap
  input "Do you want to initially clear the current config? (yes/no) ", CONTINUE$
  if lcase$(CONTINUE$) = "yes" then
    print "-> Clearing config and resetting."
    esp.waitfor("AT+RESTORE","OK",esp.waitfor.mode.match,1)
    pause 1000
    esp.waitfor("AT+CWMODE=1","OK",esp.waitfor.mode.match,1) '
    pause 400
    esp.waitfor("AT+CWDHCP_DEF=1,1","OK",esp.waitfor.mode.match,1) '
    pause 400
    print "-> esp8266 config has been cleared." : print
  end if

  pause 400
  esp.buffer.clear
  pause 200
  esp.echo.set("OFF")
  pause 200

SSID:
  input "Enter the SSID (? for a list): ", SSID$
  if SSID$ = "?" then
    print : print "-> Scanned SSIDs:"
    esp.buffer.clear
    esp.waitfor("AT+CWLAP","OK",esp.waitfor.mode.match,1)
    X% = 1
    do
      X% = instr(X%,esp.var.waitfor.buffer$, chr$(34))
      if X% = 0 then exit
      Y% = instr(X%+1,esp.var.waitfor.buffer$, chr$(34))
      if Y% = 0 then exit
      print "   ";mid$(esp.var.waitfor.buffer$, X%+1, Y%-X%-1)
      X% = Y% + 1
      ' skip mac addresses for each SSID
      X% = instr(X%,esp.var.waitfor.buffer$, chr$(34))
      if X% = 0 then exit
      Y% = instr(X%+1,esp.var.waitfor.buffer$, chr$(34))
      if Y% = 0 then exit
      X% = Y% + 1
    loop
    print
    goto SSID
  end if

  input "Enter the WIFI password: ", SSIDPW$
  if SSID$ = "" then SSID$ = "unknown"
  if SSIDPW$ = "" then SSIDPW$ = "unknown"

  do
    print
    print "-> SSID = ";SSID$
    print "-> PASS = ";SSIDPW$
    print
    input "Do you want to APPLY this configuration? (yes/no/restart) ", CONTINUE$
    if lcase$(CONTINUE$) = "restart" then goto START
    if lcase$(CONTINUE$) = "no" then goto FINISH
    if lcase$(CONTINUE$) = "yes" then exit
  loop

  esp.buffer.clear
  esp.var.timeout% = 2000
  esp.waitfor("AT+CWJAP_CUR="+chr$(34)+SSID$+chr$(34)+","+chr$(34)+SSIDPW$+chr$(34),"OK",esp.waitfor.mode.match,1)
  pause 1000
  if instr(1,esp.var.waitfor.buffer$, "CONNECTED") <> 0 then print "-> Connected to Access Point."
  if instr(1,esp.var.waitfor.buffer$, "GOT IP") <> 0 then 
    pause 300
    esp.waitfor("AT+CIFSR","OK",esp.waitfor.mode.match,1)
    X% = instr(1,esp.var.waitfor.buffer$, chr$(34))
    Y% = instr(X%+1,esp.var.waitfor.buffer$, chr$(34))
    MYIP$ = mid$(esp.var.waitfor.buffer$, X%+1, Y%-1-X%)
    print "-> IP Address is ";MYIP$
  end if
  print
  input "Do you want to SAVE this configuration? (yes/no) ", CONTINUE$
  if lcase$(CONTINUE$) = "yes" then
    print "-> Saving and resetting connection to Access Point."
    print "-> the esp will connect to this AP automatically on power-on."
    esp.waitfor("AT+CWJAP_DEF="+chr$(34)+SSID$+chr$(34)+","+chr$(34)+SSIDPW$+chr$(34),"OK",esp.waitfor.mode.match,1)
  else
    print "-> Settings are active, but will not be saved."
  end if

  pause 1000
  esp.waitfor("AT+CIPSTATUS","OK",esp.waitfor.mode.match,1)
  if instr(1,esp.var.waitfor.buffer$,"STATUS:2") <> 0 then print "-> Connected to Access Point."
    esp.waitfor("AT+CIFSR","OK",esp.waitfor.mode.match,1)
    if esp.var.return$ = "" then
    X% = instr(1,esp.var.waitfor.buffer$, chr$(34))
    Y% = instr(X%+1,esp.var.waitfor.buffer$, chr$(34))
    MYIP$ = mid$(esp.var.waitfor.buffer$, X%+1, Y%-1-X%)
    print "-> IP Address is ";MYIP$
  end if

FINISH:

  close esp.var.com.filenum%
  end

#include "esp8266lib.inc"
